import 'dart:convert';
import 'package:flutter/foundation.dart';
import 'package:http/http.dart' as http;
import 'package:jameswidget/firebase/auth/controller/firebase_auth_controller_interface.dart';

abstract class FirebaseEmailAuthController extends FirebaseAuthControllerInterface{
  Future<http.Response> connect({
    required String endPoint,
    required String apiKey,
    required String bodyData
  }) async{
    return await http.post(
      Uri.parse(endPoint+apiKey),
      headers: {'Content-Type': 'application/json'},
      body: bodyData,
    );
  }

  Map<String, dynamic> parse(http.Response response){
    const Map<String, dynamic> _errData = {"err":"fail"};
    if(response.statusCode != 200) return _errData;
    return json.decode(response.body);
  }
}

class FirebaseEMailAuthController extends FirebaseEmailAuthController{
  String email = "";
  String pw = "";
  String rePw = "";
  final String firebaseApiKey;
  bool _debugDisposed = false;

  FirebaseEMailAuthController({
    required this.firebaseApiKey,
    this.rePw = "",
  });

  String _bodyData(String email, String pw) => json.encode({"email":email, "password":pw, "returnSecureToken":true});

  @mustCallSuper
  void dispose() {
    assert(_debugAssertNotDisposed());
    assert(() {
      _debugDisposed = true;
      return true;
    }());
  }

  bool _debugAssertNotDisposed() {
    assert(() {
      if (_debugDisposed) {throw FlutterError('A $runtimeType was used after being disposed.\n''Once you have called dispose() on a $runtimeType, it can no longer be used.',);}
      return true;
    }());
    return true;
  }

  Future<Map<String, dynamic>> signIn() async{
    assert(_debugAssertNotDisposed());
    if(this.email.isEmpty) return {"err": "empty"};
    if(this.pw.isEmpty) return {"err": "empty"};
    const String _endPoint = "https://identitytoolkit.googleapis.com/v1/accounts:signInWithPassword?key=";
    try{
      return this.parse(await this.connect(endPoint: _endPoint, apiKey: this.firebaseApiKey, bodyData: this._bodyData(this.email, this.pw)));
    }
    catch(e){
      return {"error":"error"};
    }
  }

  Future<Map<String, dynamic>> signUp() async{
    assert(_debugAssertNotDisposed());
    if(this.email.isEmpty) return {"err": "empty"};
    if(this.rePw.isEmpty) return {"err": "empty"};
    if(this.pw.isEmpty) return {"err": "empty"};
    if(this.pw != this.rePw) return {"err": "notEqual"};
    const String _endPoint = "https://identitytoolkit.googleapis.com/v1/accounts:signUp?key=";
    try{
      return this.parse(await this.connect(endPoint: _endPoint, apiKey: this.firebaseApiKey, bodyData: this._bodyData(this.email, this.pw)));
    }
    catch(e){
      return {"error":"error"};
    }
  }
}
